import { Observable } from 'rxjs';
import { ObservableWithValue } from "@proscom/prostore";
/**
 * Subscribe to Observable reactively.
 * Whenever callback changes, the observable is resubscribed to, which
 * may lead to additional callback calls with the same value, because
 * some observable types call the subscriber immediately when subscription occurs.
 *
 * @param obs$ - observable to subscribe to
 * @param setState - callback to which the observable value is passed
 */
export declare function useObservable<State = any>(obs$: Observable<State> | null | undefined, setState: (state: State) => void): void;
/**
 * Subscribe to Observable with useState included.
 *
 * To ensure that no values are lost, make sure that `obs$` replays its latest value.
 * You can make observable to do so by piping it through `shareReplay` operator:
 * `obs$ = otherObs$.pipe(shareReplay())`
 *
 * @param obs$ - observable to subscribe to
 * @param initialState - initial value for the embedded state
 */
export declare function useObservableState<State = any>(obs$: Observable<State> | null | undefined, initialState: State): State;
/**
 * Subscribe to Observable with useState included.
 * Until the subscriber is called, the state value is null (on the first render).
 *
 * To ensure that no values are lost, make sure that `obs$` replays its latest value.
 * You can make observable to do so by piping it through `shareReplay` operator:
 * `obs$ = otherObs$.pipe(shareReplay())`
 *
 * @param obs$ - observable to subscribe to
 */
export declare function useObservableState<State = any>(obs$: Observable<State> | null | undefined): State | null;
/**
 * Subscribe to observable with value.
 *
 * To ensure that no values are lost, make sure that `obs$` replays its latest value.
 * You can make observable to do so by piping it through `shareReplay` operator:
 * `obs$ = otherObs$.pipe(shareReplay())`
 *
 * @param sub$ - observable with value to subscribe to
 */
export declare function useSubject<State = any>(sub$: ObservableWithValue<State>): State;
/**
 * Subscribe to observable with value.
 *
 * To ensure that no values are lost, make sure that `obs$` replays its latest value.
 * You can make observable to do so by piping it through `shareReplay` operator:
 * `obs$ = otherObs$.pipe(shareReplay())`
 *
 * @param sub$ - observable with value to subscribe to
 */
export declare function useSubject<State = any>(sub$: ObservableWithValue<State> | null | undefined): State | null;
/**
 * Subscribe to observable imperatively.
 * In contrast to simple useObservable, in this hook the observable is not
 * resubscribed to whenever the callback changes. This is more suitable
 * for subscribing to events instead of values.
 *
 * @param obs$ - observable to subscribe to
 * @param callback - callback which is called whenever the observable changes
 */
export declare function useObservableCallback<State = any>(obs$: Observable<State> | null | undefined, callback: (state: State) => void): void;
