export interface UseAsyncOperationOptions {
    finishedTimeout?: number;
    singleton?: boolean;
}
export interface UseAsyncOperationState {
    loading: number;
    finished: boolean;
}
/**
 * This error is thrown when attempt is made to call the singleton async operation
 * while previous call to it is still running.
 * Usually you should catch and swallow this error.
 *
 * @example
 *  run().then(() => {}).catch(e => {
 *    if (e instanceof AsyncSingletonError) return;
 *    toast.error(e);
 *  });
 */
export declare class AsyncSingletonError extends Error {
    constructor(m?: string);
}
/**
 * Allows to observe the status of async operation (loading, finished)
 * @param callback - async callback which performs the operation
 * @param options - hook options:
 *  - singleton - prevents calling operation until previous call is finished
 *      on the second call, rejects the promise with the instance of AsyncSingletonError
 *  - finishedTimeout - if set, after operation is completed, sets value of
 *      finished to true for the given number of milliseconds. To set finished
 *      permanently, set finishedTimeout to Infinity
 * @example
 *  const [run, loading] = useAsyncOperation(() => fetch('url').catch(e => console.error(e)));
 *  return <div>{loading ? 'Loading...' : <button onClick={run} />}
 */
export declare function useAsyncOperation<Args extends any[], Result>(callback: (...args: Args) => Promise<Result>, options?: UseAsyncOperationOptions): {
    run: (...args: Args) => Promise<Result>;
    loading: boolean;
    finished: boolean;
    setFinished: (finished: boolean) => void;
};
