"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useObservableCallback = exports.useSubject = exports.useObservableState = exports.useObservable = void 0;
var react_1 = require("react");
var ui_react_1 = require("@proscom/ui-react");
/**
 * Subscribe to Observable reactively.
 * Whenever callback changes, the observable is resubscribed to, which
 * may lead to additional callback calls with the same value, because
 * some observable types call the subscriber immediately when subscription occurs.
 *
 * @param obs$ - observable to subscribe to
 * @param setState - callback to which the observable value is passed
 */
function useObservable(obs$, setState) {
    // useLayoutEffect is used here in order to minimize probability
    // of possible emits between render and effect execution
    // so that latest emits are less likely to be lost in case obs$ is not replayed
    react_1.useLayoutEffect(function () {
        if (!obs$) {
            return;
        }
        var sub = obs$.subscribe(function (state) {
            setState(state);
        });
        return function () {
            sub.unsubscribe();
        };
    }, [obs$, setState]);
}
exports.useObservable = useObservable;
/**
 * Subscribe to Observable with useState included
 *
 * To ensure that no values are lost, make sure that `obs$` replays its latest value.
 * You can make observable to do so by piping it through `shareReplay` operator:
 * `obs$ = otherObs$.pipe(shareReplay())`
 *
 * @param obs$ - observable to subscribe to
 * @param initialState - initial value for the embedded state
 */
function useObservableState(obs$, initialState) {
    if (initialState === void 0) { initialState = null; }
    var _a = react_1.useState(initialState), state = _a[0], setState = _a[1];
    useObservable(obs$, setState);
    return state;
}
exports.useObservableState = useObservableState;
/**
 * Subscribe to observable with value.
 *
 * To ensure that no values are lost, make sure that `obs$` replays its latest value.
 * You can make observable to do so by piping it through `shareReplay` operator:
 * `obs$ = otherObs$.pipe(shareReplay())`
 *
 * @param sub$ - observable with value to subscribe to
 */
function useSubject(sub$) {
    var _a;
    return useObservableState(sub$, (_a = sub$ === null || sub$ === void 0 ? void 0 : sub$.value) !== null && _a !== void 0 ? _a : null);
}
exports.useSubject = useSubject;
/**
 * Subscribe to observable imperatively.
 * In contrast to simple useObservable, in this hook the observable is not
 * resubscribed to whenever the callback changes. This is more suitable
 * for subscribing to events instead of values.
 *
 * @param obs$ - observable to subscribe to
 * @param callback - callback which is called whenever the observable changes
 */
function useObservableCallback(obs$, callback) {
    var callbackRef = ui_react_1.useLatestCallbackRef(callback);
    useObservable(obs$, callbackRef);
}
exports.useObservableCallback = useObservableCallback;
//# sourceMappingURL=useObservable.js.map